package com.typowebguide.directory.dao;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.springframework.transaction.annotation.Transactional;

public class BaseDaoJpa<T, ID extends Serializable> implements BaseDao<T, ID> {
	@PersistenceContext protected EntityManager entityManager;
	protected Class<T> clazz;
	protected String className;
	protected String fetchAllQuery;
	
	@SuppressWarnings("unchecked")
	public BaseDaoJpa() {
		this.clazz = (Class<T>) ( (ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0];
		this.className = clazz.getSimpleName();
		this.fetchAllQuery = "SELECT a from " + this.className + " a";
	}
	
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}

	@Override
	public T findById(ID id, boolean lock) {
		return entityManager.find(clazz, id);
	}

	@Override
	public List<T> findAll() {
		return findByQuery(fetchAllQuery);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<T> findByQuery(String query) {
		return entityManager.createQuery(query).getResultList();
	}

	@Override
	@Transactional
	public T makePersistent(T entity) {
		entityManager.persist(entity);
		return entity;
	}

	@Transactional
	@Override
	public void makeTransient(T entity) {
		entityManager.remove(entity);
	}

	@Override
	public void flush() {
		entityManager.flush();
	}

	@Override
	public void clear() {
		entityManager.clear();
	}

}
